
from typing import List, Optional
from pydantic import BaseModel

from officelyTest.team_builder.nodes.filter.enums import OperatorValue
from officelyTest.team_builder.nodes.filter.enums import DataType






class IOperatorInfo(BaseModel):
    label: OperatorValue
    dataType: DataType

    # class Config:
    #     arbitrary_types_allowed = True

class ICondition(BaseModel):
    field: str
    operator: IOperatorInfo
    value: str
    logicalOperator:Optional[str] = None


    # class Config:
    #     arbitrary_types_allowed = True



class IGroupCondtions(BaseModel):
    id: str
    conditions: List[ICondition]

    # class Config:
    #     arbitrary_types_allowed = True

class IFilterObject(BaseModel):
    groups: List[IGroupCondtions]
    elseID:str

    # class Config:
    #     arbitrary_types_allowed = True



from typing import Dict, List, Tuple
from langchain_openai import ChatOpenAI
from pydantic import model_validator

from officelyTest.CustomException import MissingEnvKeys
from officelyTest.stream_handler import StreamHandler
from .llm import LLM
import os





class OpenAILLM(LLM):
    streaming:bool = True



    @property
    def base(self):
        if not os.getenv("OPENAI_API_KEY"):
            raise MissingEnvKeys("Environment variable 'OPENAI_API_KEY' is required but not set.")
        callbacks = []
        if self.tokenizer:
            self.tokenizer.model = self.model
            callbacks.append(self.tokenizer)
        is_answer = False
        if self.g:
            callbacks.append(StreamHandler(gen=self.g, with_final=False))
            is_answer = self.g.is_answer

        return ChatOpenAI(
            model=self.model, 
            temperature=self.temperature, 
            streaming=is_answer and self.streaming, 
            callbacks=callbacks
        )
    
    def with_schema(self, prompt:List[Tuple[str, str]], schema:Dict, name:str):
        self.streaming = False
        return self.base.with_structured_output(schema).with_config({"run_name":name}).invoke(prompt)
    



from officelyTest.team_builder.nodes.interface import IHistory
from officelyTest.team_builder.nodes.enums import MessageType
from officelyTest.team_builder.nodes.interface import IHistory
from officelyTest.run import invokeTeam


from abc import ABC, abstractmethod
from typing import List, Optional
from pydantic import BaseModel, Field

from officelyTest.generator import ThreadedGenerator
from ..nodes.interface import ISchema
from .enums import LLMType
from .tokenizer import Tokenizer


class LLM(BaseModel, ABC):
    """
    This is the LLM class.
    It is a subclass of BaseModel and ABC.
    """
    model:LLMType
    temperature:float = 0
    g: Optional[ThreadedGenerator] = Field(default=None)
    tokenizer: Optional[Tokenizer] = Field(default=None) 

    # class Config:
    #     arbitrary_types_allowed = True




    @property
    @abstractmethod
    def base(self):
        """
        This is the base method.
        It serves as a placeholder and does not perform any specific functionality.
        """
    


    @abstractmethod
    def with_schema(self, prompt:str, schema:List[ISchema], name:str):
        """
        This is the with_structured_output method.
        It takes in a schema and returns a response.
        """





